@@ -55,10 +55,12 @@ from solumclient.common import exc
 from solumclient.common import github
 from solumclient.common import yamlutils
 from solumclient.openstack.common.apiclient import exceptions
+from solumclient.v1 import app as cli_app
 from solumclient.v1 import assembly as cli_assem
 from solumclient.v1 import languagepack as cli_lp
 from solumclient.v1 import pipeline as cli_pipe
 from solumclient.v1 import plan as cli_plan
+from solumclient.v1 import workflow as cli_wf
 
 
 def name_is_valid(string):
@@ -245,16 +247,16 @@ Available commands:
     solum assembly list
         Print an index of all available assemblies.
 
-    solum assembly show <NAME>
+    solum assembly show <NAME|UUID>
         Print the details of an assembly.
 
-    solum assembly create <NAME> <PLAN_URI> [--description <DESCRIPTION>]
+    solum assembly create <NAME|UUID> <PLAN_URI> [--description <DESCRIPTION>]
         Create an assembly from a registered plan.
 
-    solum assembly logs <NAME>
+    solum assembly logs <NAME|UUID>
         Print an index of all operation logs for an assembly.
 
-    solum assembly delete <NAME>
+    solum assembly delete <NAME|UUID>
         Destroy an assembly.
     """
 
@@ -305,32 +307,8 @@ Available commands:
 
     def logs(self):
         """Get Logs."""
-        self.parser.add_argument('assembly',
-                                 help="Assembly uuid or name")
-        args = self.parser.parse_args()
-        assem = self.client.assemblies.find(name_or_id=args.assembly)
-        loglist = cli_assem.AssemblyManager(self.client).logs(
-            assembly_id=str(assem.uuid))
-
-        fields = ["resource_uuid"]
-        for log in loglist:
-            strategy_info = json.loads(log.strategy_info)
-            if log.strategy == 'local':
-                if 'local_storage' not in fields:
-                    fields.append('local_storage')
-                log.local_storage = log.location
-            elif log.strategy == 'swift':
-                if 'swift_container' not in fields:
-                    fields.append('swift_container')
-                if 'swift_path' not in fields:
-                    fields.append('swift_path')
-                log.swift_container = strategy_info['container']
-                log.swift_path = log.location
-            else:
-                if 'location' not in fields:
-                    fields.append('location')
-
-        self._print_list(loglist, fields)
+        print("Not Supported: Logs moved to workflows. Use  following command."
+              "\nsolum workflow logs app_id wf_id")
 
     def show(self):
         """Show an assembly's resource."""
@@ -340,7 +318,7 @@ Available commands:
         args = self.parser.parse_args()
         assemblies = self.client.assemblies.find(name_or_id=args.assembly_uuid)
         fields = ['uuid', 'name', 'description', 'status', 'application_uri',
-                  'trigger_uri', 'created_at', 'updated_at', 'workflow']
+                  'created_at', 'updated_at', 'workflow']
         self._print_dict(assemblies, fields, wrap=72)
 
 
@@ -475,9 +453,17 @@ Available commands:
         self.parser.add_argument('name',
                                  type=ValidLPName,
                                  help="Language pack name.")
+        self.parser.add_argument('--name',
+                                 type=ValidLPName,
+                                 dest='name',
+                                 help="Language pack name.")
         self.parser.add_argument('git_url',
                                  help=("Github url of custom "
                                        "language pack repository."))
+        self.parser.add_argument('--git_url',
+                                 dest='git_url',
+                                 help=("Github url of custom "
+                                       "language pack repository."))
         self.parser.add_argument('--lp_metadata',
                                  help="Language pack metadata file.")
         self.parser._names['git_url'] = 'repo URL'
@@ -535,7 +521,7 @@ Available commands:
         loglist = cli_lp.LanguagePackManager(self.client).logs(
             lp_id=str(args.lp_id))
 
-        fields = ["resource_uuid"]
+        fields = ["resource_uuid", "created_at"]
         for log in loglist:
             strategy_info = json.loads(log.strategy_info)
             if log.strategy == 'local':
@@ -557,6 +543,572 @@ Available commands:
 
 
 class AppCommands(cli_utils.CommandsBase):
+    """Commands for working with actual applications.
+
+Available commands:
+    solum app list
+        Print an index of all deployed applications.
+
+    solum app show <NAME|ID>
+        Print detailed information about one application.
+
+    solum app create [--app-file <AppFile>] [--git-url <GIT_URL>]
+                     [--lp <LANGUAGEPACK>]
+                     [--param-file <PARAMFILE>]
+                     [--setup-trigger]
+                     [--trigger-workflow <CUSTOM-WORKFLOW>]
+                      <CUSTOM-WORKFLOW>=(unittest | build | unittest+build)
+                      Without the --trigger-workflow flag,
+                      the workflow unittest+build+deploy is triggered
+                      (this is the default workflow)
+
+        Register a new application with Solum.
+
+    solum app deploy <NAME|ID>
+        Deploy an application, building any applicable artifacts first.
+
+    solum app delete <NAME|ID>
+        Delete an application and all related artifacts.
+    """
+    def _validate_app_file(self, app_data):
+        if ('workflow_config' in app_data and
+                app_data.get('workflow_config') is None):
+                msg = "Workflow config cannot be empty"
+                raise exc.CommandException(message=msg)
+        if ('trigger_actions' in app_data and
+                app_data.get('trigger_actions') is None):
+                msg = "Trigger actions cannot be empty"
+                raise exc.CommandException(message=msg)
+
+        error_message = ("Application name must be 1-100 characters and must "
+                         "only contain a-z,A-Z,0-9,-,_")
+
+        if app_data.get('name') is not None:
+            if not name_is_valid(app_data.get('name')):
+                raise exc.CommandError(message=error_message)
+
+        if 'repo_token' not in app_data:
+            app_data['repo_token'] = ''
+
+    def _get_and_validate_app_name(self, app_data, args):
+        # Check the appfile-supplied name first.
+        error_message = ("Application name must be 1-100 characters and must "
+                         "only contain a-z,A-Z,0-9,-,_")
+        app_name = ''
+        if app_data.get('name') is not None:
+            if not name_is_valid(app_data.get('name')):
+                raise exc.CommandError(message=error_message)
+            app_name = app_data.get('name')
+        # Check the arguments next.
+        elif args.name:
+            app_name = args.name
+        # Just ask.
+        else:
+            while True:
+                app_name = raw_input("Please name the application.\n> ")
+                if name_is_valid(app_name):
+                    break
+                print(error_message)
+
+        return app_name
+
+    def _get_and_validate_languagepack(self, app_data, args):
+        # Check for the language pack. Check args first, then planfile.
+        # If it's neither of those places, prompt for it and update the
+        # plan definition.
+
+        languagepack = None
+        if args.languagepack is not None:
+            languagepack = args.languagepack
+            # check if given languagepack exists or not
+            lp = None
+            try:
+                lp = self.client.languagepacks.find(name_or_id=languagepack)
+            except Exception:
+                raise exc.CommandError(
+                    "Languagepack '%s' not found." % languagepack)
+            if lp is None or lp.status != 'READY':
+                raise exc.CommandError("No languagepack in READY state. "
+                                       "Create a languagepack first.")
+            app_data['languagepack'] = languagepack
+        elif app_data.get('languagepack') is None:
+            languagepacks = self.client.languagepacks.list()
+            filtered_list = cli_utils.filter_ready_lps(languagepacks)
+
+            if len(filtered_list) > 0:
+                lpnames = [lang_pack.name for lang_pack in filtered_list]
+                lp_uuids = [lang_pack.uuid for lang_pack in filtered_list]
+                fields = ['uuid', 'name', 'description',
+                          'status', 'source_uri']
+                self._print_list(filtered_list, fields)
+                languagepack = raw_input("Please choose a languagepack from "
+                                         "the above list.\n> ")
+                while languagepack not in lpnames + lp_uuids:
+                    languagepack = raw_input("You must choose one of the named"
+                                             " language packs.\n> ")
+                app_data['languagepack'] = languagepack
+            else:
+                raise exc.CommandError("No languagepack in READY state. "
+                                       "Create a languagepack first.")
+
+    def _get_app_repo_details(self, app_data, args):
+        git_rev = 'master'
+        git_url = None
+        if (app_data.get('source') is not None and
+                app_data.get('source').get('repository') is not None):
+                    git_url = app_data['source']['repository']
+        # Commandline flag overrides stuff in the app-file
+        if args.git_url is not None:
+            git_url = args.git_url
+        # Take input from user
+        elif (app_data.get('source') is None or
+                app_data['source'].get('repository') is None or
+                app_data['source']['repository'] is ''):
+            git_url = raw_input("Please specify a git repository URL for "
+                                "your application.\n> ")
+            git_rev_i = raw_input("Please specify revision"
+                                  " (default is master).\n> ")
+            if git_rev_i is '':
+                git_rev = 'master'
+            else:
+                git_rev = git_rev_i
+
+        assert(git_url is not None)
+        assert(git_rev is not None)
+
+        git_src = dict()
+        git_src['repository'] = transform_git_url(git_url, False)
+        git_src['revision'] = git_rev
+        app_data['source'] = git_src
+
+    def _get_run_command(self, app_data, args):
+        run_cmd = None
+
+        if args.run_cmd is not None:
+            run_cmd = args.run_cmd
+        elif (app_data.get('workflow_config') is None or
+                app_data['workflow_config'].get('run_cmd') is '' or
+                app_data['workflow_config'].get('run_cmd') is None):
+            run_cmd = raw_input("Please specify start/run command for your "
+                                "application.\n> ")
+
+        if app_data.get('workflow_config') is None:
+            run_cmd_dict = dict()
+            run_cmd_dict['run_cmd'] = run_cmd
+            app_data['workflow_config'] = run_cmd_dict
+        elif (app_data['workflow_config'].get('run_cmd') is '' or
+              app_data['workflow_config'].get('run_cmd') is None):
+            app_data['workflow_config']['run_cmd'] = run_cmd
+
+    def _get_unittest_command(self, app_data, args):
+        unittest_cmd = None
+
+        if args.unittest_cmd is not None:
+            unittest_cmd = args.unittest_cmd
+            if app_data.get('workflow_config') is None:
+                unittest_cmd_dict = dict()
+                unittest_cmd_dict['test_cmd'] = unittest_cmd
+                app_data['workflow_config'] = unittest_cmd_dict
+            elif (app_data['workflow_config'].get('test_cmd') is '' or
+                  app_data['workflow_config'].get('test_cmd') is None):
+                app_data['workflow_config']['test_cmd'] = unittest_cmd
+
+    def _get_port(self, app_data, args):
+        port_list = []
+
+        if (app_data.get('ports') is None or
+                app_data['ports'] is '' or app_data['ports'] == [None]):
+            if args.port:
+                port_list.append(int(args.port))
+            else:
+                print("Using 80 as the app's default listening port")
+                port_list.append(int(80))
+
+            app_data['ports'] = port_list
+
+    def _get_parameters(self, app_data, args):
+        if args.param_file is not None:
+            try:
+                with open(args.param_file) as param_f:
+                    param_def = param_f.read()
+                app_data['parameters'] = yamlutils.load(param_def)
+            except IOError:
+                message = "Could not open param file %s." % args.param_file
+                raise exc.CommandError(message=message)
+            except ValueError:
+                message = ("Param file %s was not YAML." %
+                           args.param_file)
+                raise exc.CommandError(message=message)
+
+    def _setup_github_trigger(self, app_data, app, args):
+        # If a token is supplied, we won't need to generate one.
+        repo_token = ''
+        if hasattr(app_data, 'repo_token'):
+            repo_token = app_data['repo_token']
+
+        if args.setup_trigger or args.workflow:
+            trigger_uri = vars(app).get('trigger_uri', '')
+            if trigger_uri:
+                workflow = None
+                if args.workflow:
+                    workflow = args.workflow.replace('+', ' ').split(' ')
+                try:
+                    git_url = app_data['source']['repository']
+                    gha = github.GitHubAuth(git_url, repo_token=repo_token)
+                    gha.create_webhook(trigger_uri, workflow=workflow)
+                except github.GitHubException as ghe:
+                    raise exc.CommandError(message=str(ghe))
+
+    def create(self):
+        self.register()
+
+    def register(self):
+        """Register a new app."""
+        self.parser.add_argument('--app-file',
+                                 dest='appfile',
+                                 help="Local appfile location")
+        self.parser.add_argument('--name',
+                                 type=ValidName,
+                                 help="Application name")
+        self.parser.add_argument('--languagepack',
+                                 help='Language pack')
+        self.parser.add_argument('--lp',
+                                 dest='languagepack',
+                                 help='Language pack')
+        self.parser.add_argument('--git-url',
+                                 dest='git_url',
+                                 help='Source repo')
+        self.parser.add_argument('--run-cmd',
+                                 dest='run_cmd',
+                                 help="Application entry point")
+        self.parser.add_argument('--unittest-cmd',
+                                 dest='unittest_cmd',
+                                 help="Command to execute unit tests")
+        self.parser.add_argument('--port',
+                                 dest="port",
+                                 type=ValidPort,
+                                 help="The port your application listens on")
+        self.parser.add_argument('--param-file',
+                                 dest='param_file',
+                                 help="A yaml file containing custom"
+                                      " parameters to be used in the"
+                                      " application")
+        self.parser.add_argument('--setup-trigger',
+                                 action='store_true',
+                                 dest='setup_trigger',
+                                 help="Set up app trigger on git repo")
+
+        trigger_help = ("Which of stages build, unittest, deploy to trigger "
+                        "from git. For example: "
+                        "--trigger-workflow=unittest+build+deploy. "
+                        "Implies --setup-trigger.")
+        self.parser.add_argument('--trigger-workflow',
+                                 default='',
+                                 dest='workflow',
+                                 help=trigger_help)
+
+        args = self.parser.parse_args()
+        app_data = None
+        if args.appfile is not None:
+            try:
+                with open(args.appfile, 'r') as inf:
+                    app_data = yamlutils.load(inf.read())
+                    self._validate_app_file(app_data)
+            except Exception as exp:
+                raise exc.CommandException(str(exp))
+        else:
+            app_data = {
+                'version': 1,
+                'description': 'default app description.',
+                'source': {
+                    'repository': '',
+                    'revision': 'master'
+                },
+                'workflow_config': {
+                    'test_cmd': '',
+                    'run_cmd': ''
+                },
+                'trigger_actions': ['build', 'deploy'],
+                'repo_token': ''
+            }
+
+        # app file schema
+        schema = {
+            "title": "app file schema",
+            "type": "object",
+            "properties": {
+                "version": {
+                    "type": "integer"
+                },
+                "name": {
+                    "type": "string"
+                },
+                "description": {
+                    "type": "string"
+                },
+                "languagepack": {
+                    "type": "string"
+                },
+                "source": {
+                    "type": "object"
+                },
+                "workflow_config": {
+                    "type": "object"
+                },
+                "repo_token": {
+                    "type": "string"
+                },
+                "trigger_actions": {
+                    "type": "array"
+                },
+                "ports": {
+                    "type": "array"
+                }
+            },
+            "required": ["version", "name", "description",
+                         "languagepack", "source", "workflow_config",
+                         "trigger_actions", "ports"]
+        }
+
+        app_name = self._get_and_validate_app_name(app_data, args)
+        app_data['name'] = app_name
+
+        self._get_and_validate_languagepack(app_data, args)
+
+        self._get_app_repo_details(app_data, args)
+
+        self._get_run_command(app_data, args)
+
+        self._get_unittest_command(app_data, args)
+
+        self._get_port(app_data, args)
+
+        self._get_parameters(app_data, args)
+
+        #  TODO(vijendar): currently doing very basic validation.
+        # Need to implement more robust schema based validation
+        appdata_keys = app_data.keys()
+        appdata_keys.sort()
+        schema_keys = schema['properties'].keys()
+        schema_keys.sort()
+        if appdata_keys != schema_keys:
+            message = "Unknown key(s) in app data file: %s" % list(
+                set(appdata_keys) - set(schema_keys))
+            raise exc.CommandError(message=message)
+
+        app = self.client.apps.create(**app_data)
+
+        self._setup_github_trigger(app_data, app, args)
+
+        app.trigger = app.trigger_actions
+        app.workflow = app.workflow_config
+
+        fields = ['name', 'id', 'created_at', 'description', 'languagepack',
+                  'ports', 'source', 'workflow',
+                  'trigger_uuid', 'trigger', 'trigger_uri']
+        self._print_dict(app, fields, wrap=72)
+
+    def update(self):
+        """Update the registration of an existing app."""
+        self.parser.add_argument('app')
+
+        self.parser.add_argument('--name', type=str)
+        self.parser.add_argument('--desc', type=str)
+        self.parser.add_argument('--lp', type=str)
+        self.parser.add_argument('--ports', type=str)
+        self.parser.add_argument('--source.repo', dest='source_repo', type=str)
+        self.parser.add_argument('--source.rev', dest='source_rev', type=str)
+        self.parser.add_argument('--test_cmd', type=str)
+        self.parser.add_argument('--run_cmd', type=str)
+        self.parser.add_argument('--trigger', type=str)
+
+        args = self.parser.parse_args()
+
+        app = self.client.apps.find(name_or_id=args.app)
+
+        to_update = {}
+        if args.name:
+            to_update['name'] = args.name
+        if args.desc:
+            to_update['description'] = args.desc
+        if args.lp:
+            to_update['languagepack'] = args.lp
+        if args.ports:
+            ports = args.ports.strip('[]').replace(',', ' ')
+            ports = [int(p, 10) for p in ports.split(' ') if p]
+            to_update['ports'] = ports
+        if args.source_repo:
+            to_update['source'] = to_update.get('source', {})
+            to_update['source']['repository'] = args.source_repo
+        if args.source_rev:
+            to_update['source'] = to_update.get('source', {})
+            to_update['source']['revision'] = args.source_rev
+        if args.test_cmd:
+            to_update['workflow_config'] = to_update.get('trigger', {})
+            to_update['workflow_config']['test_cmd'] = args.test_cmd
+        if args.run_cmd:
+            to_update['workflow_config'] = to_update.get('trigger', {})
+            to_update['workflow_config']['run_cmd'] = args.run_cmd
+        if args.trigger:
+            trigger = args.trigger.strip('[]').replace(',', ' ').split(' ')
+            to_update['trigger_actions'] = trigger
+
+        if not to_update:
+            raise exc.CommandException(message="Nothing to update")
+
+        updated_app = self.client.apps.patch(app_id=app.id, **to_update)
+
+        updated_app.trigger = updated_app.trigger_actions
+        updated_app.workflow = updated_app.workflow_config
+
+        fields = ['name', 'id', 'created_at', 'description', 'languagepack',
+                  'entry_points', 'ports', 'source', 'workflow',
+                  'trigger_uuid', 'trigger']
+        self._print_dict(updated_app, fields, wrap=72)
+
+    def list(self):
+        """List all apps."""
+        apps = self.client.apps.list()
+        fields = ['name', 'id', 'created_at', 'description', 'languagepack']
+        self._print_list(apps, fields)
+
+    def show(self):
+        """Show details of one app."""
+        self.parser.add_argument('name')
+        args = self.parser.parse_args()
+        app = self.client.apps.find(name_or_id=args.name)
+
+        app.trigger = app.trigger_actions
+        app.workflow = app.workflow_config
+
+        fields = ['name', 'id', 'created_at', 'description', 'languagepack',
+                  'entry_points', 'ports', 'source',
+                  'trigger_uuid', 'trigger', 'app_url']
+        self._print_dict(app, fields, wrap=72)
+
+        wfman = cli_wf.WorkflowManager(self.client, app_id=app.id)
+        wfs = wfman.list()
+        fields = ['wf_id', 'id', 'status']
+        print("'%s' workflows and their status:" % args.name)
+        self._print_list(wfs, fields)
+
+    def delete(self):
+        """Delete an app."""
+        self.parser.add_argument('name')
+        args = self.parser.parse_args()
+        app = self.client.apps.find(name_or_id=args.name)
+        cli_app.AppManager(self.client).delete(
+            app_id=str(app.id))
+
+    def _create_workflow(self, actions):
+        self.parser.add_argument('name')
+        args = self.parser.parse_args()
+        app = self.client.apps.find(name_or_id=args.name)
+        wf = (cli_wf.WorkflowManager(self.client,
+                                     app_id=app.id).create(actions=actions))
+        fields = ['wf_id', 'app_id', 'actions', 'config',
+                  'source', 'id', 'created_at', 'updated_at']
+        self._print_dict(wf, fields, wrap=72)
+
+    def unittest(self):
+        """Create a new workflow for an app."""
+        actions = ['unittest']
+        self._create_workflow(actions)
+
+    def build(self):
+        """Create a new workflow for an app."""
+        actions = ['unittest', 'build']
+        self._create_workflow(actions)
+
+    def deploy(self):
+        """Create a new workflow for an app."""
+        actions = ['unittest', 'build', 'deploy']
+        self._create_workflow(actions)
+
+
+class WorkflowCommands(cli_utils.CommandsBase):
+    """Commands for working with workflows.
+
+Available commands:
+
+    solum workflow list <APP_NAME|UUID>
+        List all application workflows.
+
+    solum workflow show <APP_NAME|UUID> <WORKFLOW_ID|UUID>
+        Print the details of a workflow.
+
+    solum workflow logs <APP_NAME|UUID> <WORKFLOW_ID|UUID>
+        List all the logs of a given workflow.
+    """
+
+    def list(self):
+        """Show all of an app's live workflows."""
+        self.parser.add_argument('app')
+        args = self.parser.parse_args()
+        app = self.client.apps.find(name_or_id=args.app)
+        wfs = cli_wf.WorkflowManager(self.client, app_id=app.id).list()
+        fields = ['wf_id', 'id', 'actions', 'status',
+                  'created_at', 'updated_at']
+        self._print_list(wfs, fields)
+
+    def show(self):
+        """Show one of an app's live workflows."""
+        # Either "solum workflow show <app_id_or_name> <workflow_uuid>
+        # Or "solum workflow show <app_id_or_name> <workflow_revision>
+        self.parser.add_argument('app')
+        self.parser.add_argument('workflow')
+        args = self.parser.parse_args()
+        revision = args.workflow
+        try:
+            revision = int(revision, 10)
+        except ValueError:
+            revision = args.workflow
+        app = self.client.apps.find(name_or_id=args.app)
+
+        wfman = cli_wf.WorkflowManager(self.client, app_id=app.id)
+        wf = wfman.find(revision_or_id=revision)
+        fields = ['wf_id', 'app_id', 'actions', 'config',
+                  'source', 'id', 'created_at', 'updated_at', 'status']
+        self._print_dict(wf, fields, wrap=72)
+
+    def logs(self):
+        """Get Logs."""
+        self.parser.add_argument('app',
+                                 help="App uuid or name")
+        self.parser.add_argument('workflow',
+                                 help="Workflow id or uuid")
+        args = self.parser.parse_args()
+        revision = args.workflow
+        try:
+            revision = int(revision, 10)
+        except ValueError:
+            revision = args.workflow
+        app = self.client.apps.find(name_or_id=args.app)
+
+        wfman = cli_wf.WorkflowManager(self.client, app_id=app.id)
+        loglist = wfman.logs(revision_or_id=revision)
+        fields = ["resource_uuid"]
+        for log in loglist:
+            strategy_info = json.loads(log.strategy_info)
+            if log.strategy == 'local':
+                if 'local_storage' not in fields:
+                    fields.append('local_storage')
+                log.local_storage = log.location
+            elif log.strategy == 'swift':
+                if 'swift_container' not in fields:
+                    fields.append('swift_container')
+                if 'swift_path' not in fields:
+                    fields.append('swift_path')
+                log.swift_container = strategy_info['container']
+                log.swift_path = log.location
+            else:
+                if 'location' not in fields:
+                    fields.append('location')
+
+        self._print_list(loglist, fields)
+
+
+class OldAppCommands(cli_utils.CommandsBase):
     """Commands for working with applications.
 
 Available commands:
@@ -606,14 +1158,6 @@ Available commands:
             if not name_is_valid(plan_definition.get('name')):
                 raise exc.CommandError(message=error_message)
 
-    def _filter_ready_lps(self, lp_list):
-        filtered_list = []
-        for lp in lp_list:
-            if lp.status == 'READY':
-                filtered_list.append(lp)
-
-        return filtered_list
-
     def list(self):
         """Print a list of all deployed applications."""
         # This is just "plan list".
@@ -768,13 +1312,13 @@ Available commands:
                                 lp1 = (
                                     self.client.languagepacks.find
                                     (name_or_id=lp)
-                                    )
+                                )
                             except Exception as e:
                                 if type(e).__name__ == 'NotFound':
                                     raise exc.CommandError("Languagepack %s "
                                                            "not registered"
                                                            % lp)
-                            filtered_list = self._filter_ready_lps([lp1])
+                            filtered_list = cli_utils.filter_ready_lps([lp1])
                             if len(filtered_list) <= 0:
                                 raise exc.CommandError("Languagepack %s "
                                                        "not READY" % lp)
@@ -796,7 +1340,7 @@ Available commands:
                     'artifact_type': 'heroku',
                     'name': '',
                     'content': {},
-                    }]}
+                }]}
 
         # NOTE: This assumes the plan contains exactly one artifact.
 
@@ -832,7 +1376,7 @@ Available commands:
             plan_definition['artifacts'][0]['language_pack'] = languagepack
         elif plan_definition['artifacts'][0].get('language_pack') is None:
             languagepacks = self.client.languagepacks.list()
-            filtered_list = self._filter_ready_lps(languagepacks)
+            filtered_list = cli_utils.filter_ready_lps(languagepacks)
 
             if len(filtered_list) > 0:
                 lpnames = [lang_pack.name for lang_pack in filtered_list]
@@ -990,8 +1534,7 @@ Available commands:
         assembly = self.client.assemblies.create(name=plan.name,
                                                  description=plan.description,
                                                  plan_uri=plan.uri)
-        fields = ['uuid', 'name', 'description', 'status', 'application_uri',
-                  'trigger_uri']
+        fields = ['uuid', 'name', 'status', 'application_uri']
         self._print_dict(assembly, fields, wrap=72)
 
     def delete(self):
@@ -1147,21 +1690,52 @@ Available commands:
     solum lp delete <NAME|UUID>
         Destroy a language pack.
 
-    solum lp logs <UUID>
+    solum lp logs <NAME|UUID>
         Show logs for a language pack.
 
-
     solum app help
         Show a help message specific to app commands.
 
     solum app list
         Print an index of all deployed applications.
 
-    solum app show <NAME|UUID>
+    solum app show <APP_NAME|UUID>
         Print detailed information about one application.
 
+    solum app create [--app-file <AppFile>] [--git-url <GIT_URL>]
+                     [--lp <LANGUAGEPACK>]
+                     [--param-file <PARAMFILE>]
+                     [--setup-trigger]
+                     [--trigger-workflow <CUSTOM-WORKFLOW>]
+                      <CUSTOM-WORKFLOW>=(unittest | build | unittest+build)
+                      Without the --trigger-workflow flag,
+                      the workflow unittest+build+deploy is triggered
+                      (this is the default workflow)
 
-    solum app create [--plan-file <PLANFILE>] [--git-url <GIT_URL>]
+        Register a new application with Solum.
+
+    solum app deploy <APP_NAME|UUID>
+        Deploy an application, building any applicable artifacts first.
+
+    solum app delete <APP_NAME|UUID>
+        Delete an application and all related artifacts.
+
+    solum app logs <APP_NAME|UUID>
+        Show the logs of an application for all the deployments.
+
+    solum workflow list <APP_NAME|UUID>
+        List all application workflows.
+
+    solum workflow show <APP_NAME|UUID> <WORKFLOW_ID|UUID>
+        Print the details of a workflow.
+
+    solum workflow logs <APP_NAME|UUID> <WORKFLOW_ID|UUID>
+        List all the logs of a given workflow.
+
+
+    SOON TO BE DEPRECATED:
+
+    solum oldapp create [--plan-file <PLANFILE>] [--git-url <GIT_URL>]
                      [--lp <LANGUAGEPACK>] [--run-cmd <RUN_CMD>]
                      [--unittest-cmd <UNITTEST_CMD>]
                      [--name <NAME>] [--port <PORT>]
@@ -1172,22 +1746,10 @@ Available commands:
                      [--trigger-workflow <WORKFLOW>]
         Register a new application with Solum.
 
-    solum app deploy <NAME|UUID>
-        Deploy an application, building any applicable artifacts first.
-
-    solum app delete <NAME|UUID>
-        Delete an application and all related artifacts.
-
-    solum app logs <NAME|UUID>
-        Show the logs of an application for all the deployments.
-
-
-    SOON TO BE DEPRECATED:
-
     solum assembly list
         Print an index of all available assemblies.
 
-    solum assembly create <NAME> <PLAN_URI> [--description <DESCRIPTION>]
+    solum assembly create <NAME|UUID> <PLAN_URI> [--description <DESCRIPTION>]
         Create an assembly from a registered plan.
 
     solum assembly delete <NAME|UUID>
@@ -1197,6 +1759,7 @@ Available commands:
     parser = PermissiveParser()
 
     resources = {
+        'oldapp': OldAppCommands,
         'app': AppCommands,
         'plan': PlanCommands,
         'assembly': AssemblyCommands,
@@ -1204,8 +1767,9 @@ Available commands:
         'lp': LanguagePackCommands,
         'languagepack': LanguagePackCommands,
         'component': ComponentCommands,
-
         'info': InfoCommands,
+        'wf': WorkflowCommands,
+        'workflow': WorkflowCommands,
     }
 
     choices = resources.keys()
@@ -1218,6 +1782,10 @@ Available commands:
                         dest='show_version',
                         help="Report solum version.")
 
+    parser.add_argument('-E', '--show-errors', dest='show_errors',
+                        action='store_true',
+                        help='Debug. Show traceback on error.')
+
     parsed, _ = parser.parse_known_args()
 
     if parsed.show_version:
@@ -1227,13 +1795,16 @@ Available commands:
     resource = vars(parsed).get('resource')
 
     if resource in resources:
-        try:
+        if parsed.show_errors:
             resources[resource](parser)
-        except Exception as e:
-            if hasattr(e, 'message'):
-                print("ERROR: %s" % e.message)
-            else:
-                print("ERROR: %s" % e)
+        else:
+            try:
+                resources[resource](parser)
+            except Exception as e:
+                if hasattr(e, 'message'):
+                    print("ERROR: %s" % e.message)
+                else:
+                    print("ERROR: %s" % e)
 
     else:
         print(main.__doc__)
